unit ferns_wind;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls,
  matrix;

type

  { TFernForm }

  TFernForm = class(TForm)
    FernProb1: TTrackBar;
    FernProb2: TTrackBar;
    FernProb3: TTrackBar;
    Fern1Mid: TTrackBar;
    Fern2Mid: TTrackBar;
    Fern3Mid: TTrackBar;
    Fern4Mid: TTrackBar;
    Fern1Tr: TTrackBar;
    Fern2Tr: TTrackBar;
    Fern3Tr: TTrackBar;
    Fern4Tr: TTrackBar;
    procedure DrawPoint (const x, y, r, g, b : Real);
    procedure FernChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  FernForm: TFernForm;

implementation

{$R *.lfm}

const
   epochs = 5000;
   IFS_VectorsNum = 4;
   IFS_Barnsley_fern_probs : array[0..2] of Real = (0.02, 0.15, 0.13);
   IFS_fern_1: array[0..2,0..2] of double = ((0, 0, 0.5),
                                             (0, 0.27, 0),
                                             (0, 0, 1));
   IFS_fern_2: array[0..2,0..2] of double = ((-0.14, 0.26, 0.57),
                                             (0.25, 0.22, -0.04),
                                             (0, 0, 1));
   IFS_fern_3: array[0..2,0..2] of double = ((0.17, -0.21, 0.41),
                                             (0.22, 0.18, 0.09),
                                             (0, 0, 1));
   IFS_fern_4: array[0..2,0..2] of double = ((0.78, 0.03, 0.11),
                                             (-0.03, 0.74, 0.27),
                                             (0, 0, 1));

procedure TFernForm.DrawPoint (const x, y, r, g, b : Real);
begin
   Assert ((0 <= r) and (r <= 1) and (0 <= g) and (g <= 1)
           and (0 <= b) and (b <= 1),
           'ChaosGameSDL.DrawPoint: RGB component outside 0..1 range');
   if (0 <= x) and (x < 1) and (0 <= y) and (y < 1)
   then begin
     Canvas.Brush.Color:=
       RGBToColor(round ($ff * r), round ($ff * g), round ($ff * b));
     Canvas.FillRect(trunc (x * WIDTH)-1, HEIGHT - trunc (y * HEIGHT) - 2,
                     trunc (x * WIDTH)+1, HEIGHT - trunc (y * HEIGHT));
   end;
end;

procedure TFernForm.FernChange(Sender: TObject);
begin
  Repaint;
end;

{ TFernForm }

procedure TFernForm.FormCreate(Sender: TObject);
begin

end;

procedure TFernForm.FormPaint(Sender: TObject);
var
   {We use an N+1 dimensional matrix representation
    of affine transformations.}
   IFS_Vectors : array[0..IFS_VectorsNum-1] of Tmatrix3_double;
   {The last vector probability is: 1 - sum of the others.}
   IFS_probs : array[0..IFS_VectorsNum-2] of Real;
   r, c : Real;
   t, branch : Integer;
   point : Tvector3_double;
begin
   IFS_probs := IFS_Barnsley_fern_probs;
   IFS_probs[0] *= FernProb1.Position * 2 / 10;
   IFS_probs[1] *= FernProb2.Position * 2 / 10;
   IFS_probs[2] *= FernProb3.Position * 2 / 10;
   IFS_Vectors[0].data := IFS_fern_1;
   IFS_Vectors[1].data := IFS_fern_2;
   IFS_Vectors[2].data := IFS_fern_3;
   IFS_Vectors[3].data := IFS_fern_4;
   IFS_Vectors[0].data[1,1] *= Fern1Mid.Position * 2 / 10;
   IFS_Vectors[1].data[1,1] *= Fern2Mid.Position * 2 / 10;
   IFS_Vectors[2].data[1,1] *= Fern3Mid.Position * 2 / 10;
   IFS_Vectors[3].data[1,1] *= Fern4Mid.Position * 2 / 10;
   IFS_Vectors[0].data[1,2] *= Fern1Tr.Position * 2 / 10;
   IFS_Vectors[1].data[1,2] *= Fern2Tr.Position * 2 / 10;
   IFS_Vectors[2].data[1,2] *= Fern3Tr.Position * 2 / 10;
   IFS_Vectors[3].data[1,2] *= Fern4Tr.Position * 2 / 10;
   point.init (0, 0, 1);
   c := 1;
   for t := 1 to epochs do
   begin
      r := random; c := 0.6*c + 0.4*r; branch := 0;
      while (branch < Length (IFS_probs))
            and (IFS_probs[branch] < r) do
      begin
         r := r - IFS_probs[branch];
         Inc (branch)
      end;
      point := IFS_Vectors[branch] * point;
      {Print the point with a fancy color.}
      DrawPoint (point.data[0], point.data[1], c, (c + 1) / 2, 1 - c)
   end;

end;

end.

