unit mineboard;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  Buttons, StdCtrls;

type
  TFieldButton = TSpeedButton;
  TFieldState = (EmptyField, N1, N2, N3, N4, N5, N6, N7, N8,
                 Hidden, Flagged, Bomb);
  TField = class
  private
    StateValue: TFieldState;
    HasBomb: Boolean;
    procedure SetState(newState: TFieldState);
  public
    Button: TFieldButton;
    X,Y: Integer;
    Neighbors: array[0..7] of TField;
    property State: TFieldState read StateValue write SetState;
    constructor Create(Panel: TPanel; aButton: TFieldButton;
                       aX,aY: Integer; aBomb: Boolean);
    procedure FieldFlagClick(Sender: TObject);
    procedure FieldCheckClick(Sender: TObject);
    procedure SetMouseDown(Sender: TObject; MouseButton: TMouseButton;
                           _sh : TShiftState; _x, _y : Integer);
  end;

  { TForm1 }
  TBoard = array[0..10, 0..10] of TField;
  TForm1 = class(TForm)
    BitBtn1: TBitBtn;
    Panel: TPanel;
    SelectDirectoryDialog1: TSelectDirectoryDialog;
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    Board: TBoard;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

constructor TField.Create(Panel: TPanel; aButton: TFieldButton;
                          aX,aY: Integer; aBomb: Boolean);
var i: Integer;
begin
  Button := aButton;
  X := aX; Y := aY;
  State := Hidden;
  HasBomb := aBomb;
  Button.Left := X * 20;
  Button.Top := Y * 20;
  Button.Height := 22;
  Button.Width := 22;
  Button.Parent := Panel;
  Button.Down := False;
  Button.OnMouseDown := @self.SetMouseDown;
  Button.OnClick := @self.FieldCheckClick;
  for i:=0 to 7 do Neighbors[i] := nil;
end;

procedure TField.SetState(newState: TFieldState);
var Pixmap : TPixmap; Dir: String;
begin
  StateValue := newState;
  if (newState = Hidden) or (newState = Flagged)
  then begin
    Button.Flat := False;
    Button.Enabled := True;
  end
  else begin
    Button.Flat := True;
    Button.Enabled := False;
  end;
  Pixmap := TPixmap.Create;
  Dir := Form1.SelectDirectoryDialog1.FileName;
  if State = Flagged then Pixmap.LoadFromFile(Dir+'/flag.xpm')
  else if State = Bomb
  then Pixmap.LoadFromFile(Dir+'/bombface6.xpm')
  else if (State <> Hidden) and (State <> EmptyField)
  then Pixmap.LoadFromFile(Dir+'/' + IntToStr(Ord(State)) + '.xpm');
  if (State <> Hidden) and (State <> EmptyField)
  then Button.Glyph.Assign(Pixmap)
  else Button.Glyph.Assign(nil);
  {TODO: Pixmap.Free?}
end;

procedure TField.FieldFlagClick(Sender: TObject);
begin
  if State = Flagged then State := Hidden
  else if State = Hidden then State := Flagged;
end;

procedure TField.FieldCheckClick(Sender: TObject);
var i: Integer;
  Bombs: Integer = 0;
begin
  if HasBomb then begin State := Bomb; Exit end;
  for i:=0 to 7 do
    if (Neighbors[i] <> nil) and Neighbors[i].HasBomb then Inc(Bombs);
  State := TFieldState(Bombs);
end;

procedure TField.SetMouseDown(Sender: TObject; MouseButton: TMouseButton;
                              _sh: TShiftState; _x,_y: Integer);
begin
  if MouseButton = mbRight then FieldFlagClick(Sender);
end;

procedure TForm1.FormCreate(Sender: TObject);
var i,j : Integer;
  PNG: TPortableNetworkGraphic;
  Directory: String;
begin
  SelectDirectoryDialog1.Execute;
  Directory := SelectDirectoryDialog1.FileName;
  PNG := TPortableNetworkGraphic.Create;
  PNG.LoadFromFile(Directory+'/win.png');
  BitBtn1.Glyph.Assign(PNG);
  {BitBtn1.Glyph.LoadFromFile(Directory + '/ball.bmp');}
  Randomize;
  for i := 0 to 10 do
    for j := 0 to 10 do
      Board[i,j] := TField.Create (Panel, TFieldButton.Create(Self),
                                   i, j, Random < 0.15);
  for i := 0 to 10 do
    for j := 0 to 10 do
      begin
        if i > 0 then Board[i,j].Neighbors[0] := Board[i-1,j];
        if (i > 0) and (j > 0) then Board[i,j].Neighbors[1] := Board[i-1,j-1];
        if j > 0 then Board[i,j].Neighbors[2] := Board[i,j-1];
        if i < 10 then Board[i,j].Neighbors[3] := Board[i+1,j];
        if (i < 10) and (j < 10) then Board[i,j].Neighbors[4] := Board[i+1,j+1];
        if j < 10 then Board[i,j].Neighbors[5] := Board[i,j+1];
        if (i > 0) and (j < 10) then Board[i,j].Neighbors[6] := Board[i-1,j+1];
        if (i < 10) and (j > 0) then Board[i,j].Neighbors[7] := Board[i+1,j-1];
      end;

end;

end.

